#ifndef _EEPROM_H
#define _EEPROM_H

#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

#ifndef CY_TYPES
#define CY_TYPES
#if defined(__alpha__)
typedef unsigned long	ucdouble;	/* 64 bits, unsigned */
typedef unsigned int	uclong;		/* 32 bits, unsigned */
#else
typedef unsigned long	uclong;		/* 32 bits, unsigned */
#endif
typedef unsigned short	ucshort;	/* 16 bits, unsigned */
typedef unsigned char	ucchar;		/* 8 bits, unsigned */
#endif /* CY_TYPES */


#ifndef writeb
#define writeb(port,val)	(*(volatile ucchar *)(port) = (ucchar)(val))
#define writew(port,val)	(*(volatile ucshort *)(port) = (ucshort)(val))
#define writel(port,val)	(*(volatile uclong *)(port) = (uclong)(val))

#define readb(port)		(*(volatile ucchar *)(port))
#define readw(port)		(*(volatile ucshort *)(port))
#define readl(port)		(*(volatile uclong *)(port))
#endif

enum EEPROM_TYPE {EEPROM_PLX9050, EEPROM_PLX9060};

//  These are the bit values for the PLX Runtime register 
//  (offset 0x6E for 9060, offset 0x52 for 9050)
#define CLOCK_BIT	(1<<8)
#define CS_BIT		(1<<9)
#define DOUT_BIT	(1<<10)
#define DIN_BIT		(1<<11)
#define	PLX_9060_EEPROM_ENABLE_BIT	(1<<0)

//  Enable the EEPROM
#define plx9050_eeprom_enable(ppp)					\
		writew((ppp-2),(readw(ppp-2) & 0xFFC3) | 0x0030)
#define plx9060_eeprom_enable(ppp)					\
		writew((ppp),readw(ppp) & ~PLX_9060_EEPROM_ENABLE_BIT)

#define	eeprom_delay()							\
	    {								\
		static int	delay_count;				\
		static char	delay_buf1[128];			\
		static char	delay_buf2[128];			\
		for (delay_count = 0 ; delay_count < 100 ; delay_count++)    \
			memcpy (delay_buf1, delay_buf2, sizeof(delay_buf1)); \
	    }

// #define eeprom_enable(ppp)  plx9050_eeprom_enable(ppp)
#define eeprom_enable(ppp,type)						\
	    {								\
		if (type==EEPROM_PLX9060)				\
			plx9060_eeprom_enable(ppp);			\
		else							\
			plx9050_eeprom_enable(ppp);			\
	    }


// Generate a rising edge on the clock line
#define clock_tick(ppp)							\
	{	clock_out(ppp,0); eeprom_delay();			\
		clock_out(ppp,1); eeprom_delay();			\
		clock_out(ppp,0);			}

// Output a specific value on the clock line
#define clock_out(ppp,value)						\
		 writew((ppp),					\
			(readw(ppp)&~CLOCK_BIT) | ((value)?CLOCK_BIT:0))

// Get the value of the data input line
#define data_in(ppp)  ((readw(ppp)&DIN_BIT)?1:0)

// Output a value on the data output and chip select lines
#define data_out(ppp,value)						\
		 writew((ppp),					\
			(readw(ppp)&~DOUT_BIT) | ((value)?DOUT_BIT:0))
#define chip_select(ppp,value)						\
		 writew((ppp),					\
			(readw(ppp)&~CS_BIT) | ((value)?CS_BIT:0))
#define output_bit(ppp,value)						\
		 { data_out(ppp,value); clock_tick(ppp); }
#define output_word(ppp,value,n_bits,tmp)				\
		for(tmp = n_bits - 1 ; tmp >= 0 ; tmp--)		\
			output_bit((ppp),(value)&(1<<(tmp)))

//  This is a 2K bits device (256 bytes)
#define	ADDR_BITS	(6)

// Function declarations
int eeprom_ewen (unsigned long, int);
int eeprom_ewds (unsigned long, int);
int eeprom_read (unsigned long, unsigned int, unsigned short *, int);
int eeprom_write (unsigned long, unsigned int, unsigned short, int);
int eeprom_read_block (unsigned long, unsigned int, void *, size_t, int, int);
int eeprom_write_block (unsigned long, unsigned int, void *, size_t, int, int);

#endif /* _EEPROM_H */

